/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { mkdir, stat, writeFile } from "node:fs/promises";
import { homedir } from "node:os";
import { join } from "node:path";
import { createInterface } from "node:readline/promises";
import type { Config } from "./types.ts";

void (async(): Promise<void> => {
  const reader = createInterface({
    input:  process.stdin,
    output: process.stdout,
  });

  const configDirectory = join(homedir(), ".config");
  const appconfigDirectory = join(configDirectory, "naomis-drpc");
  const directoryExists = await stat(appconfigDirectory).catch(() => {
    return null;
  });
  if (!directoryExists?.isDirectory()) {
    await mkdir(appconfigDirectory, { recursive: true });
  }
  const configFile = join(appconfigDirectory, "config.json");

  process.stdout.write(`Config path: ${configFile}\n`);

  const configExists = await stat(configFile).catch(() => {
    return null;
  });
  if (configExists?.isFile()) {
    const confirmOverwrite = await reader.question(
      "Config file already exists. Overwrite? (y/N) ",
    );
    if (confirmOverwrite.toLowerCase() !== "y") {
      process.stdout.write("Declined to overwrite config file. Exiting...\n");
      process.exit(0);
    }
  }
  const config: Partial<Config> = {};

  process.stdout.write("Welcome to the Discord Rich Presence Setup Wizard!\n");
  process.stdout.write(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "Before you get started, you need to create a Discord application on their developer portal. https://discord.dev\n",
  );
  process.stdout.write(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "The name you assign your new application will show up in your Discord STATUS when you are connected with this tool.",
  );
  const appId = await reader.question(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "Enter your Discord application ID. You can obtain this from the Discord Developer Portal:",
  );
  config.appId = appId;
  const details = await reader.question(
    "Enter the first line of text you'd like to show in your presence:",
  );
  config.details = details;
  const state = await reader.question(
    "Enter the second line of text you'd like to show in your presence:",
  );
  config.state = state;
  process.stdout.write(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "The next section sets up the images to display in your presence. You can upload these images to your application under Rich Presence -> Art Assets.\n",
  );
  const largeImageKey = await reader.question(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "Enter the DISCORD name of the image you uploaded to display as the large image:",
  );
  config.largeImageKey = largeImageKey;
  const largeImageText = await reader.question(
    "Enter the text to display when hovering over the large image:",
  );
  config.largeImageText = largeImageText;
  const smallImageKey = await reader.question(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "Enter the DISCORD name of the image you uploaded to display as the small image:",
  );
  config.smallImageKey = smallImageKey;
  const smallImageText = await reader.question(
    "Enter the text to display when hovering over the small image:",
  );
  config.smallImageText = smallImageText;
  process.stdout.write(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "You can optionally display buttons that other users can click on in your presence.\n",
  );
  const wantsButtons = await reader.question(
    "Do you want to display buttons in your presence? (y/N) ",
  );
  if (wantsButtons.toLowerCase() === "y") {
    const buttonOneLabel = await reader.question(
      "Enter the text to display on the first button:",
    );
    config.buttonOneLabel = buttonOneLabel;
    const buttonOneUrl = await reader.question(
      "Enter the URL to open when the first button is clicked:",
    );
    config.buttonOneUrl = buttonOneUrl;
    const buttonTwoLabel = await reader.question(
      "Enter the text to display on the second button:",
    );
    config.buttonTwoLabel = buttonTwoLabel;
    const buttonTwoUrl = await reader.question(
      "Enter the URL to open when the second button is clicked:",
    );
    config.buttonTwoUrl = buttonTwoUrl;
  }

  await writeFile(configFile, JSON.stringify(config, null, 4));
  process.stdout.write(
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    "Config file written successfully! You can now run naomis-drpc to start your Discord Rich Presence.\n",
  );
  process.exit(0);
})();
